"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComposeService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const BootstrapUtils_1 = require("./BootstrapUtils");
const path_1 = require("path");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const fs = require('fs');
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
const targetNodesFolder = BootstrapUtils_1.BootstrapUtils.targetNodesFolder;
const targetDatabasesFolder = BootstrapUtils_1.BootstrapUtils.targetDatabasesFolder;
const targetGatewaysFolder = BootstrapUtils_1.BootstrapUtils.targetGatewaysFolder;
class ComposeService {
    constructor(root, params) {
        this.root = root;
        this.params = params;
    }
    async run(passedPresetData) {
        const presetData = passedPresetData !== null && passedPresetData !== void 0 ? passedPresetData : BootstrapUtils_1.BootstrapUtils.loadExistingPresetData(this.params.target);
        const currentDir = process.cwd();
        const target = path_1.join(currentDir, this.params.target);
        const targetDocker = path_1.join(target, `docker`);
        if (this.params.reset) {
            BootstrapUtils_1.BootstrapUtils.deleteFolder(targetDocker);
        }
        const dockerFile = path_1.join(targetDocker, 'docker-compose.yml');
        if (fs.existsSync(dockerFile)) {
            logger.info(dockerFile + ' already exist. Reusing. (run -r to reset)');
            return BootstrapUtils_1.BootstrapUtils.loadYaml(dockerFile);
        }
        await BootstrapUtils_1.BootstrapUtils.mkdir(targetDocker);
        await BootstrapUtils_1.BootstrapUtils.generateConfiguration(presetData, path_1.join(this.root, 'config', 'docker'), targetDocker);
        const user = await BootstrapUtils_1.BootstrapUtils.resolveDockerUserFromParam(this.params.user);
        const vol = (hostFolder, imageFolder) => {
            return hostFolder + ':' + imageFolder;
        };
        logger.info(`creating docker-compose.yml from last used profile.`);
        const services = {};
        const resolvePorts = (internalPort, openPort) => {
            if (!openPort) {
                return [];
            }
            if (openPort === true || openPort === 'true') {
                return [`${internalPort}:${internalPort}`];
            }
            return [`${openPort}:${internalPort}`];
        };
        const resolveService = async (rawService) => {
            if (false) {
                // POC about creating custom aws images.
                const serviceName = rawService.container_name;
                const volumes = rawService.volumes || [];
                const image = rawService.image;
                const repository = 'nem-repository';
                const dockerfileContent = `FROM docker.io/${image}\n\n${volumes
                    .map((v) => {
                    const parts = v.split(':');
                    return `ADD ${parts[0].replace('../', '').replace('./', 'docker/')} ${parts[1]}`;
                })
                    .join('\n')}\n`;
                const dockerFile = path_1.join(target, 'Dockerfile-' + serviceName);
                await BootstrapUtils_1.BootstrapUtils.writeTextFile(dockerFile, dockerfileContent);
                await Promise.all(volumes.map(async (v) => {
                    const parts = v.split(':');
                    await BootstrapUtils_1.BootstrapUtils.mkdir(path_1.join(targetDocker, parts[0]));
                }));
                const generatedImageName = repository + ':' + serviceName;
                await BootstrapUtils_1.BootstrapUtils.createImageUsingExec(target, dockerFile, generatedImageName);
                // const awsUserId = '172617417348';
                // aws ecr get-login-password --region us-east-1 | docker login --username AWS --password-stdin 172617417348.dkr.ecr.us-east-1.amazonaws.com
                // const absoluteImageUrl = `${awsUserId}.dkr.ecr.us-east-1.amazonaws.com/${generatedImageName}`;
                // await BootstrapUtils.exec(`docker tag ${generatedImageName} ${absoluteImageUrl}`);
                return Object.assign(Object.assign({}, rawService), { image: generatedImageName, volumes: undefined });
            }
            else {
                return Object.assign(Object.assign({}, rawService), { user });
            }
        };
        await Promise.all((presetData.databases || []).map(async (n) => {
            services[n.name] = await resolveService({
                container_name: n.name,
                image: presetData.mongoImage,
                command: `bash -c "/bin/bash /userconfig/mongors.sh ${n.name} & mongod --dbpath=/dbdata --bind_ip=${n.name}"`,
                stop_signal: 'SIGINT',
                ports: resolvePorts(27017, n.openPort),
                volumes: [vol(`./mongo`, `/userconfig/:ro`), vol(`../${targetDatabasesFolder}/${n.name}`, '/dbdata:rw')],
            });
        }));
        const nodeWorkingDirectory = '/symbol-workdir';
        const nodeCommandsDirectory = '/symbol-commands';
        await Promise.all((presetData.nodes || []).map(async (n) => {
            const nodeService = await resolveService({
                container_name: n.name,
                image: presetData.symbolServerImage,
                command: `bash -c "/bin/bash ${nodeCommandsDirectory}/runServerRecover.sh  ${n.name} && /bin/bash ${nodeCommandsDirectory}/startServer.sh ${n.name}"`,
                stop_signal: 'SIGINT',
                working_dir: nodeWorkingDirectory,
                restart: 'on-failure:2',
                ports: resolvePorts(7900, n.openPort),
                volumes: [vol(`../${targetNodesFolder}/${n.name}`, nodeWorkingDirectory), vol(`./server`, nodeCommandsDirectory)],
            });
            nodeService.depends_on = [];
            if (n.host) {
                nodeService.hostname = n.host;
                nodeService.networks = { default: { aliases: [n.host] } };
            }
            services[n.name] = nodeService;
            if (n.brokerHost) {
                services[n.brokerHost] = await resolveService({
                    container_name: n.brokerHost,
                    image: nodeService.image,
                    working_dir: nodeWorkingDirectory,
                    command: `bash -c "/bin/bash ${nodeCommandsDirectory}/runServerRecover.sh ${n.brokerHost} && /bin/bash ${nodeCommandsDirectory}/startBroker.sh ${n.brokerHost}"`,
                    ports: resolvePorts(7902, n.openBrokerPort),
                    stop_signal: 'SIGINT',
                    restart: 'on-failure:2',
                    volumes: nodeService.volumes,
                });
                nodeService.depends_on.push(n.brokerHost);
            }
        }));
        await Promise.all((presetData.gateways || []).map(async (n) => {
            const gatewayService = await resolveService({
                container_name: n.name,
                image: presetData.symbolRestImage,
                command: 'npm start --prefix /app/catapult-rest/rest /symbol-workdir/rest.json',
                stop_signal: 'SIGINT',
                working_dir: nodeWorkingDirectory,
                ports: resolvePorts(3000, n.openPort),
                volumes: [vol(`../${targetGatewaysFolder}/${n.name}`, nodeWorkingDirectory)],
                depends_on: [n.databaseHost],
            });
            if (n.ipv4_address) {
                gatewayService.networks = {
                    default: {
                        ipv4_address: n.ipv4_address,
                    },
                };
            }
            services[n.name] = gatewayService;
        }));
        const dockerCompose = {
            version: '3',
            services: services,
        };
        if (presetData.subnet)
            dockerCompose.networks = {
                default: {
                    ipam: {
                        config: [
                            {
                                subnet: presetData.subnet,
                            },
                        ],
                    },
                },
            };
        await BootstrapUtils_1.BootstrapUtils.writeYaml(dockerFile, dockerCompose);
        logger.info(`docker-compose.yml file created ${dockerFile}`);
        return dockerCompose;
    }
}
exports.ComposeService = ComposeService;
ComposeService.defaultParams = {
    target: 'target',
    user: BootstrapUtils_1.BootstrapUtils.CURRENT_USER,
    reset: false,
};
